# frozen_string_literal: true
require 'json'
module Hermes
  module Api
    module Dmo
      # checker Api::Dmo::HermesController avant d'ajouter des methodes
      class LessorsController < Hermes::Api::Dmo::ApplicationController
        set_instance_for :objectives

        class << self
          def mutate_params
            {
              affiliate: { type: 'String'},
              api_id: { type: 'String'},
              id_cristal: { type: 'String' },
              id_rex: { type: 'String' },
              rpls_id: { type: 'String'},
              siret: { type: 'String' },
              title: { type: 'String' },
            }
          end 
          
          def acceptable_params
            {
              lessor_objective_ids: { type: 'Array' },
              siren: { type: 'String' },
            }.merge(self.mutate_params).merge(super)
          end 
      
          def valid_operators
            %w[$in $nin $ne $gt $gte $lt $lte $eq $regex]
          end

          def regex_params
            {
              title: {type: 'String'},
              siret: {type: 'String'},
              siren: {type: 'String'},
            }
          end 
        end

        def index
          acceptable_params *self.class.acceptable_params.keys
          regex_params *self.class.regex_params.keys
          valid_operators *self.class.valid_operators
          super
        end

        def objective_handle_params
          acceptable_params *Hermes::Api::Dmo::LessorObjectivesController.acceptable_params.keys
          regex_params *Hermes::Api::Dmo::LessorObjectivesController.regex_params.keys
          valid_operators *Hermes::Api::Dmo::LessorObjectivesController.valid_operators
        end
        
        def admin_search
          return render json: {status: "unauthorized", code: 401, message: "user is no superadmin"} unless is_super_admin

          aggregation = Hermes::UserRight.all

          if !params['lessor_params'].blank?
            lessor_query = admin_search_param(
              params: params['lessor_params'], 
              possible_params: [:id, :id_cristal, :siren, :siret],
              partial_regex_params: [:title]
            )
            lessor_hint = lessor_query["title"] ? 'title_1' : nil
            aggregation = aggregation.where(:lessor_id.in => Hermes::Lessor.where(lessor_query).hint(lessor_hint).pluck(:id)) 
          end 

          if !params['user_params'].blank?
            user_query = admin_search_param(
              params: params['user_params'], 
              possible_params: [:id, :email, :auth_roles],
              full_regex_params: [:first_name, :last_name]
            )
            user_hint = nil
            user_hint = 'first_name_1' if user_query["first_name"]
            user_hint = 'last_name_1' if user_query["last_name"]
            aggregation = aggregation.where(:user_id.in => User.where(user_query).hint(user_hint).pluck(:id)) 
          end 

          @options[:no_attributes] = true

          paginate(
            aggregation,
            each_serializer: Hermes::AdminSearchRightSerializer,
            **@options
          )
        end

        def paginate_objectives(objectives)
          objective_handle_params
          objectives = objectives.where(build_query(Hermes::LessorObjective))
                      .order(order_params)
          Rails.logger.debug "TEST #{objectives.selector}"
          paginate(
            objectives,
            each_serializer: Hermes::LessorObjectiveSerializer
          )
        end

        def all_objectives
          objectives = @instance.lessor_objectives
          paginate_objectives(objectives)
        end

        def active_objectives
          objectives = @instance.lessor_objectives.actives
          paginate_objectives(objectives)
        end

        def switch_manual_mode
          if is_super_admin
            lessor = Hermes::Lessor.find(params[:id])
            if lessor.switch_manual_mode
              render json: {status: "success", code: 200, message: "manual mode switch to #{lessor.manual_mode}"}
            else
              render json: {status: "not found", code: 404, message: 'lessor not found'}
            end 
          else 
            render json: {status: "unauthorized", code: 401, message: "user is not a superadmin"}
          end 
        end 

        def current_operation_names
          operation_names = @instance.current_operation_names
          render :json => operation_names
        end

        def add_user
          if @instance.writable_by? current_user
            user = ::User.find params[:data][:attributes][:user_id]
            if user && user.update(lessor_id: @instance.id) 
              return render json: { data: 'user updated' }
            end
          end
          
          render_json_error(:not_found, :object_not_found) unless @instance
        end

        def remove_user
          if @instance.writable_by? current_user
            user = ::User.find params[:data][:attributes][:user_id]
            if user && user.update(lessor_id: nil) 
              return render json: { data: 'user updated' }
            end
          end
          
          render_json_error(:not_found, :object_not_found) unless @instance
        end

        def rdsl
          return render_json_error(:not_found, :object_not_found) unless @instance

          user_ids = Hermes::UserRight.where(lessor_id: @instance.id, right: 'administrator').pluck(:user_id)
          users = User.where(:id.in => user_ids, auth_roles: 'rdsl')

          paginate(
            users,
            each_serializer: Hermes::UserSerializer,
            **@options
          )
        end
      end
    end
  end
end
