# frozen_string_literal: true

module Hermes
  class UserRight
    include Hermes::Concerns::CommonFieldsConcern
    field :right, type: String
    field :type, type: String
    field :title, type: String
    field :group_id, type: String
    field :user_email, type: String
    field :user_first_name, type: String
    field :user_last_name, type: String
    field :user_phone_number, type: String
    field :user_auth_roles, type: Array
    field :user_gender, type: String, default: 'M'
    field :origin, type: String, default: 'front'
    field :role, type: String
    
    common_indexes *%i[
      right
      type
      title
      group_id
      user_email
      user_first_name
      user_last_name
      user_phone_number
      user_auth_roles
      user_gender
      origin
      role
    ]

    index(user_id: 1, lessor_id: 1)
    index(user_id: 1, entity_id: 1)
    index(user_id: 1, group_id: 1)
    index(user_id: 1, type: 1, entity_id: 1, lessor_id: 1)

    belongs_to :entity, index: true, class_name: 'Hermes::Entity'
    belongs_to :lessor, index: true, class_name: 'Hermes::Lessor'
    belongs_to :user, index: true, class_name: 'User'

    validates_presence_of :right, :type
    validates_presence_of :entity_id, if: -> {type == 'entity'}
    validates_presence_of :lessor_id, if: -> {type == 'lessor'}
    validates_inclusion_of :right, in: ["reader", "user", "administrator"]
    validates_inclusion_of :type, in: ["entity", "lessor"]

    before_save :before_save_actions
    after_create :after_create_actions
    after_save :after_save_actions
    after_destroy :after_destroy_actions

    validate :user_id_not_changed
    validate :setup_user_and_create_it
    validate :already_has_right

    def already_has_right
      if ur = Hermes::UserRight.find_by(:id.ne => id,user_id: user_id, type: type, entity_id: entity_id, lessor_id: lessor_id)
        errors.add(:user_id, "This user already have a right on this #{ur.type} #{ur.right}")
      end
    end 

    def user_id_not_changed
      if user_id_changed? && self.user_id_was
        errors.add(:user_id, "Cannot change user for an existing user_right!")
      end
    end

    def setup_user_and_create_it
      return unless user_id.blank?
      user = User.find_by(email: user_email.downcase)

      unless user
        [:user_first_name, :user_last_name, :user_auth_roles ].each do |att|
          self.errors.add(att, "user params missing") if self[att].blank?
        end 
        return if self.errors.messages.length > 0

        user = User.new(get_user_params.merge(password: Devise.friendly_token[0,20]))
        
        if !user.valid?
          user.errors.messages.each do |att, error|
            self.errors.add("user_#{att}", error.first)
          end 
          return
        else
          user.save!
          if user.instance_variable_get(:@create_user_response) && ![200, 201, 409].include?(user.instance_variable_get(:@create_user_response).code)
            errors.add(:user_id, "beiam error when creating : #{user.instance_variable_get(:@create_user_response).code} : #{user.instance_variable_get(:@create_user_response).body}")
          end 
        end 
      end 
      self.user_id = user&.id
    end 

    def get_user_params
      email = user_email.downcase
      {
        username: email,
        first_name: user_first_name,
        last_name: user_last_name,
        email: email,
        gender: user_gender,
        phone_number: user_phone_number,
        login: email,
        auth_roles: user_auth_roles,
        provider: HERMES_CONFIG['identity_provider_key'],
        create_user_data: {
          gender: user_gender,
          lastName: user_last_name,
          firstName: user_first_name,
          email: email,
          login: email,
          mobileNumber: user_phone_number,
          roles: user_auth_roles,
        },
      }
    end


    def before_save_actions
      self.group_id = get_group_id.to_s
      self.title = (type == 'entity') ? entity.title : lessor.title
    end

    def after_create_actions
      update_user_group_ids
    end

    def after_destroy_actions
      update_user_group_ids
    end

    def after_save_actions
      update_user_group_ids if group_id_changed?
      update_user_auth_roles if user_auth_roles_changed? && (user.auth_roles & user_auth_roles).empty?
    end

    def update_user_auth_roles
      user.update(auth_roles: ((user.auth_roles || [] ) + user_auth_roles ).uniq) 
    end 

    def other_user_right_with_role(role)
      Hermes::UserRight.find_by(:id.ne => id, user_id: user_id, user_auth_roles: role)
    end 

    def get_group_id
      rights_mapping = {
        reader: 'group_id',
        user: 'supervisor_group_id',
        administrator: 'manager_group_id',
      }

      entity = case type
      when 'entity'
        Hermes::Entity.find(entity_id)
      when 'lessor'
        Hermes::Lessor.find(lessor_id)
      end
      entity && entity[rights_mapping[right.to_sym]]
    end

    def update_user_group_ids
      group_ids = get_user_group_ids(user_id)
      user.set(updated_at: DateTime.current, group_ids: group_ids) if user
    end


    def get_user_group_ids(user_id)
      Hermes::UserRight.where(user_id: user_id).pluck(:group_id).compact.uniq
    end
  end
end
