require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::LessorContactsController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers

  check_mutate_params(
    model: Hermes::LessorContact, 
    mutate_params: Hermes::Api::Dmo::LessorContactsController.mutate_params, 
    factory: :lessor_contact,
    min_params: {
      name: 'test'
    },
    other_factory:[
      {factory: :lessor, id: 'lessor1'},
      {factory: :lessor, id: 'lessor2'},
    ],
  )

  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:user) { create(:user) }
  let!(:lessor) { create(:lessor, siret: '00000000000000', readers: [user.id.to_s]) }
  let!(:lessor2) { create(:lessor, siret: '00000000000000', readers: [responsible.id.to_s]) }
  
  let!(:lessor_contact1) { create(:lessor_contact, name: 'name1', lessor_id: lessor2.id.to_s) }
  let!(:lessor_contact2) { create(:lessor_contact, name: 'name2', lessor_id: lessor2.id.to_s) }
  let!(:lessor_contact3) { create(:lessor_contact, name: 'test3', lessor_id: lessor2.id.to_s) }
  let!(:lessor_contact4) { create(:lessor_contact, name: 'test4', lessor_id: lessor.id.to_s) }
  

  describe 'POST /lessor_contacts' do
    describe 'Create as admin' do
      before :each do
        authenticated_header(request, responsible)
        post 'create', {
            data: {
                attributes: {
                    name: 'test'
                }
            }
        }
      end
      it 'returns status code 201' do
        expect(response).to have_http_status(201)
      end
    end

    describe 'Create as user without lessor_id' do
      before :each do
        authenticated_header(request, user)
        post 'create', {
            data: {
                attributes: {
                    name: 'test'
                }
            }
        }
      end
      it 'returns error when no lessor_id' do
        expect(response).to have_http_status(422)
      end
    end

    describe 'Create as user with lessor_id' do
      before :each do
        authenticated_header(request, user)
        post 'create', {
            data: {
                attributes: {
                    name: 'test',
                    lessor_id: lessor.id.to_s
                }
            }
        }
      end
      it 'returns 201' do
        expect(response).to have_http_status(201)
      end
    end
  end

  describe 'GET /lessor_contacts' do
    describe 'as admin' do 
      before :each do
        authenticated_header(request, responsible)
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(4)
      end
    end
    
    describe 'filter by lessor_id' do 
      before :each do
        authenticated_header(request, responsible)
        get 'index', {lessor_id: lessor2.id.to_s}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
      end
    end
    describe 'as user' do 
      before :each do
        authenticated_header(request, user)
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(1)
      end
    end

    describe 'regex on name' do 
      before :each do
        authenticated_header(request, responsible)
        get 'index', {name: {"$regex": "test" }}
      end
      it 'returns a valid json' do
        expect(response).to have_http_status(200)
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(2)
      end
    end
    
  end

  describe 'GET /lessor_contacts/:id' do
    describe 'verif it work' do
      before :each do
        authenticated_header(request, user)
        get 'show', {id: lessor_contact4.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end

    describe 'verif it work not work with a non authorized user' do

      before :each do
        authenticated_header(request, user)
        get 'show', {id: lessor_contact1.id}
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(404)
      end
    end
  end

  describe 'DELETE /lessor_contacts/:id' do
    before :each do
      authenticated_header(request, responsible)

      delete 'destroy', {
          id: lessor_contact1.id.to_s
      }
    end
    it 'returns status code 204' do
      expect(response).to have_http_status(204)
    end
  end

end
require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::LessorContactsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_LESSOR_CONTACTS'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }

  path '/api/dmo/lessor_contacts' do
    get 'Retrieves all lessor contacts' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::LessorContactsController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'lessor contacts found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:lessor_contacts) { create_list(:lessor_contact, 5, name: 'test') }

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) { lessor_contacts }
        end
      end
    end
  end

  path '/api/dmo/lessor_contacts' do
    post 'Create lessor contact' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::LessorContactsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  name: 'test' }
        )
      end

      response '201', 'lessor contact created' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'lessor_contacts'
        end
      end
    end
  end

  path '/api/dmo/lessor_contacts/{id}' do
    get 'Retrieves a single lessor_contact by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 
      
      response '200', 'lessor_contact found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:lessor_contact) {create(:lessor_contact, name: 'test')}
        let(:id) {lessor_contact.id}

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {lessor_contact}
        end
      end
    end
  end

  path '/api/dmo/lessor_contacts/{id}' do
    put 'Update lessor contact' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::LessorContactsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter name: :id, in: :path, type: :string
      let(:id) { create(:lessor_contact, name: 'test', user: admin).id }
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  name: 'test' }
        )
      end

      response '200', 'lessor contact updated' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)
          
          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'lessor_contacts'
        end
      end
    end
  end

  path '/api/dmo/lessor_contacts/{id}' do
    delete 'Delete a single lessor contact by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '204', 'lessor contact deleted' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { create(:lessor_contact, name: 'test').id }
        run_test!
      end
    end
  end

end
