require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
require 'vcr_config'

RSpec.describe BatchTasks::SynchroRex, type: :service do

  describe '#update_contact' do
    5.times do |i|
      let!(:"entity#{i+1}") { create(:entity, id_cristal: "#{i+1}", siret: "0000000000000#{i+1}") }
    end 

    let!(:user) { create(:user, email: 'deborah.meliani@yopmail.com') }
    let!(:user_right1) { create(:user_right, user: user, type: 'entity', entity_id: entity5.id, right: 'administrator') }
    let!(:rex_entity_deborah) do 
      {
        "fields" =>  {
          "nom" =>  "MELIANI",
          "rolecrelocal" =>  "CRE",
          "id_cristal" =>  "1",
          "prenom" =>  "DEBORAH",
          "email" =>  "deborah.meliani@yopmail.com",
          "id_etablissement_cristal" =>  [
            "1",
            "3",
            "4",
          ],
        },
      }
    end 

    describe 'Existing user' do
      subject { described_class.new.update_contact(rex_entity_deborah) }

      it 'update user' do
        subject

        expect(user.reload.first_name).to eq "DEBORAH"
        expect(user.reload.auth_roles).to eq ["collaborateur offre logement", "cre"]
        expect(user.reload.id_cristal).to eq "1"
        expect(User.all().count).to eq(1)
      end

      it 'update access' do 
        expect(Hermes::Entity.writable_by(user).find_by(id: entity5.id)).not_to be_nil  

        subject

        user.reload
        expect(Hermes::Entity.writable_by(user).count).to eq(4)  
        expect(Hermes::Entity.writable_by(user).find_by(id: entity1.id)).not_to be_nil  
        expect(Hermes::Entity.writable_by(user).find_by(id: entity2.id)).to be_nil
        expect(Hermes::Entity.writable_by(user).find_by(id: entity3.id)).not_to be_nil  
        expect(Hermes::Entity.writable_by(user).find_by(id: entity4.id)).not_to be_nil  
        expect(Hermes::Entity.writable_by(user).find_by(id: entity5.id)).not_to be_nil  
      end 

      it 'update cre_email' do 
        entity5.set(updated_at: DateTime.current, cre_email: user.email)

        subject

        user.reload
        expect(entity1.reload.cre_email).to eq(user.email)
        expect(entity1.reload.cre_first_name).to eq("DEBORAH")
        expect(entity1.reload.cre_last_name).to eq("MELIANI")
        expect(entity2.reload.cre_email).to eq(nil)  
        expect(entity3.reload.cre_email).to eq(user.email)  
        expect(entity4.reload.cre_email).to eq(user.email)  
        expect(entity5.reload.cre_email).to eq(nil)  
      end 

      it 'when already as right on entity' do
        ur2 = create(:user_right, user: user, type: 'entity', entity_id: entity1.id, right: 'reader')

        subject
        
        user.reload
        expect(Hermes::Entity.writable_by(user).find_by(id: entity1.id)).not_to be_nil  
        expect(entity1.reload.cre_email).to eq(user.email)
        expect(ur2.reload.right).to eq('administrator')
      end 
    end

    describe 'Not existing user' do
      it 'create user' do
        rex_entity = {
          "fields" =>  {
            "nom" =>  "Doe",
            "id_etablissement_cristal" =>  [
              "5",
            ],
            "rolecrelocal" =>  "CRE",
            "id_cristal" =>  "2",
            "prenom" =>  "John",
            "email" =>  "john.doe@yopmail.com"
          },
        }
        described_class.new.update_contact(rex_entity)

        created_user = User.find_by(email: 'john.doe@yopmail.com')
        expect(User.all().count).to eq(2)
        expect(created_user.first_name).to eq "John"
        expect(Hermes::Entity.writable_by(user).count).to eq(1)  
        expect(Hermes::Entity.writable_by(created_user).find_by(id: entity5.id)).not_to be_nil
      end
    end

    describe 'not removing cre national' do
      let!(:user_right_national1) { create(:user_right, user_id: user.id, entity_id: entity1.id, right: 'administrator', type: 'entity', role: 'cre_national')}
      let!(:user_right_national3) { create(:user_right, user_id: user.id, entity_id: entity3.id, right: 'administrator', type: 'entity', role: 'cre_national')}

      it 'works' do
        user_right1.update(role: 'cre_national')
        entity5.update(cre_email: user.email)
        described_class.new.update_contact(rex_entity_deborah)

        user.reload
        expect(Hermes::Entity.writable_by(user).count).to eq(4)  
        expect(Hermes::UserRight.find_by(user_id: user.id, entity_id: entity1.id).role).to eq('cre_national')
        expect(entity1.reload.cre_email).to eq(user.email)
        expect(entity5.reload.cre_email).to be_nil
      end 
    end 
  end
  
  describe '#update_entity' do 
    let!(:entity) { create(:entity, title: 'MON AGENCE', id_cristal: '119057') }
    let!(:entity_parent) { create(:entity, title: 'MON AGENCE PARENT', id_cristal: '299785') }

    let!(:rex_entity_json) do 
      {
        "id": "73acd55b-e64d-4de2-bc25-3ae63e70ad70",
        "idEntityDefinition": "a0c7c1dc-435a-47c8-a8ff-0ccc4396588b",
        "idRefExt": nil,
        "fields": {
          "date_fin_activite": nil,
          "grpmontantcollecte": "2489072",
          "categorie": nil,
          "entadressecodpos": "33610",
          "id_parent_rex": "",
          "liste_login": "",
          "entadressevoie3": nil,
          "siret": "70198020300726",
          "entadressevoie1": "34 Rue Antoine Primat",
          "rso": "SUEZ RV SUD OUEST",
          "sigle": "SUEZ RV SUD OUEST",
          "entadressevoie2": "CS 60257",
          "id_absorbant_cristal": "",
          "anneecollecte": "2019",
          "id_parent_cristal": "299785",
          "id_root_cristal": "299785",
          "id_root_rex": "",
          "id_cristal": "119057",
          "entmontantcollecte": "117282",
          "id_absorbant_rex": "",
          "entadressevoie": "Chez SUEZ RV CSP",
          "entadresseville": "CANEJAN",
          "app_comm": "SUEZ RV SUD OUEST",
          "itiephycrenational": "000001",
          "lprenomcrenational": "John", 
          "lnomcrenational": "Doe", 
          "ladrmailcrenational": "johndoe@test.com", 
        },
        "externalData": nil,
        "traceability": {
          "createdBy": "idrex_test_user_user_rw",
          "createdDate": "2020-04-01T12:40:26.196+0000",
          "lastModifiedBy": "idrex_test_user_user_rw",
          "lastModifiedDate": "2020-04-01T12:40:26.196+0000"
        }
      }
    end 

    let!(:rex_parent_entity_json) do 
      {
        "id": "73acd55b-e64d-4de2-bc25-3ae63e70ad70",
        "idEntityDefinition": "a0c7c1dc-435a-47c8-a8ff-0ccc4396588b",
        "idRefExt": nil,
        "fields": {
          "date_fin_activite": nil,
          "grpmontantcollecte": "2489072",
          "categorie": nil,
          "entadressecodpos": "33610",
          "id_parent_rex": "",
          "liste_login": "",
          "entadressevoie3": nil,
          "siret": "70198020300726",
          "entadressevoie1": "34 Rue Antoine Primat",
          "rso": "SCNF",
          "sigle": "SCNF",
          "entadressevoie2": "CS 60257",
          "id_absorbant_cristal": "",
          "anneecollecte": "2019",
          "id_parent_cristal": nil,
          "id_root_cristal": "299785",
          "id_root_rex": "",
          "id_cristal": "299785",
          "entmontantcollecte": "117282",
          "id_absorbant_rex": "",
          "entadressevoie": "Chez SUEZ RV CSP",
          "entadresseville": "CANEJAN",
          "app_comm": "SCNF",
          "itiephycrenational": "000001",
          "lprenomcrenational": "John", 
          "lnomcrenational": "Doe", 
          "ladrmailcrenational": "johndoe@test.com", 
        },
        "externalData": nil,
        "traceability": {
          "createdBy": "idrex_test_user_user_rw",
          "createdDate": "2020-04-01T12:40:26.196+0000",
          "lastModifiedBy": "idrex_test_user_user_rw",
          "lastModifiedDate": "2020-04-01T12:40:26.196+0000"
        }
      }
    end 

    let!(:rex_contact_json) do
      {
        "id": '65baf92c-1c34-4287-bbd0-661a9246529e',
        "idEntityDefinition": '864e35c1-862a-4e8e-8f13-deab5ccdbe7',
        "idRefExt": nil,
        "fields": {
          "code_postal": '',
          "tel_pro": '',
          "ville": '',
          "id_etablissement_rex": '',
          "nom": "BENHLIMAx",
          "id_etablissement_cristal": ['170'],
          "rolecrelocal": 'CRE',
          "civ": '',
          "id_bailleur_rex": '',
          "login_iam": '',
          "id_bailleur_cristal": '',
          "fonction": 'COMMERCIAL',
          "adresse": '',
          "id_cristal": '000001',
          "fax": '',
          "prenom": 'MOHAMED',
          "email": 'mohamed.benhlima@yopmail.com'
        },
        "externalData": nil,
        "traceability": {
          "createdBy": 'idrex_test_user_user_rw',
          "createdDate": '2020-04-03T03:41:24.233+0000',
          "lastModifiedBy": 'username',
          "lastModifiedDate": '2020-05-18T13:19:17.908+0000'
        }
      }
    end 

    describe 'Existing entity' do
      it 'update entity and with parent' do
        described_class.new.update_entity(rex_entity_json, 'Hermes::Entity')
        expect(entity.reload.title).to eq "SUEZ RV SUD OUEST"
        expect(Hermes::Entity.all().count).to eq(2)
        expect(Hermes::Entity.find_by(id: entity.reload.parent_id).title).to eq("MON AGENCE PARENT")
      end
    end

    describe 'Not existing entity' do
      it 'create entity and parent' do
        entity.destroy
        entity_parent.destroy
        object = described_class.new
        allow(object.client).to receive(:get_token).and_return("token")
        allow(object.client).to receive(:search_structures).and_return([rex_parent_entity_json])

        object.update_entity(rex_entity_json, 'Hermes::Entity')
        created_entity = Hermes::Entity.find_by(id_cristal: '119057')
        expect(created_entity.reload.title).to eq "SUEZ RV SUD OUEST"
        expect(Hermes::Entity.all().count).to eq(2)
        expect(Hermes::Entity.find_by(id: created_entity.reload.parent_id).title).to eq("SCNF")
      end
    end

    describe 'cre national' do
      let!(:user2) { create(:user, id_cristal: '000002') }
      let!(:object) { described_class.new }

      subject do
        object.update_entity(rex_entity_json, 'Hermes::Entity')
      end
      
      before do 
        allow(object.client).to receive(:get_token).and_return("token")
        allow(object.client).to receive(:search_structures).and_return([rex_parent_entity_json])
        allow(object.client).to receive(:search_contacts).and_return([rex_contact_json])
      end 

      describe 'new cre national with new user' do
        it 'works' do 
          subject
          
          user = User.find_by(id_cristal: '000001')
          user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
          expect(user_right.user_id).to eq(user.id)
          expect(entity.reload.cre_national_id_cristal).to eq('000001')
          expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
          expect(entity.reload.cre_national_first_name).to eq('John')
          expect(entity.reload.cre_national_last_name).to eq('Doe')
        end 
      end 

      describe 'new cre national with existing user matching with id cristal' do
        let!(:user) { create(:user, email: 'test@olo.com', id_cristal: '000001', first_name: 'John', last_name: 'Doe', password: '12345678', password_confirmation: '12345678')}

        it 'works' do
          subject

          
          user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
          expect(user_right.user_id).to eq(user.id)
          expect(entity.reload.cre_national_id_cristal).to eq('000001')
          expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
          expect(entity.reload.cre_national_first_name).to eq('John')
          expect(entity.reload.cre_national_last_name).to eq('Doe')

        end 
      end 

      describe 'new cre national with existing user matching with email' do
        let!(:user) { create(:user, email: 'johndoe@test.com', first_name: 'John', last_name: 'Doe', password: '12345678', password_confirmation: '12345678')}

        it 'works' do
          subject

          user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
          expect(user_right.user_id).to eq(user.id)
          expect(entity.reload.cre_national_id_cristal).to eq('000001')
          expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
          expect(entity.reload.cre_national_first_name).to eq('John')
          expect(entity.reload.cre_national_last_name).to eq('Doe')
          expect(user.reload.id_cristal).to eq('000001')
        end
      end 

      describe 'updating cre national' do
        let!(:user_right) {create(:user_right, user_id: user2.id, entity_id: entity.id, role: 'cre_national', right: 'administrator', type: 'entity')}
        
        it 'works' do
          entity.update(cre_national_id_cristal: '000002', cre_national_email: user2.email)

          subject

          user = User.find_by(id_cristal: '000001')
          user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
          expect(user_right.user_id).to eq(user.id)
          expect(entity.reload.cre_national_id_cristal).to eq('000001')
          expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
          expect(entity.reload.cre_national_first_name).to eq('John')
          expect(entity.reload.cre_national_last_name).to eq('Doe')
          expect(entity.readable_by?(user)).to eq(true)
          expect(entity.readable_by?(user2)).to eq(false)
        end 
      end 

      describe 'already cre' do
        let!(:user) { create(:user, email: 'johndoe@test.com', id_cristal: '000001', first_name: 'John', last_name: 'Doe', password: '12345678', password_confirmation: '12345678')}
        let!(:user_right_local) { create(:user_right, user_id: user.id, entity_id: entity.id, right: 'administrator', type: 'entity')}
        
        it 'works' do
          subject

          user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
          expect(user_right.user_id).to eq(user.id)
          expect(entity.reload.cre_national_id_cristal).to eq('000001')
          expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
          expect(entity.reload.cre_national_first_name).to eq('John')
          expect(entity.reload.cre_national_last_name).to eq('Doe')
          expect(Hermes::UserRight.where(entity_id: entity.id, user: user).count).to eq(1)
        end
      end 

      describe 'remove cre national' do
        let!(:user) { create(:user, id_cristal: '000001')}
        let!(:user_right) {create(:user_right, user_id: user.id, entity_id: entity.id, role: 'cre_national', right: 'administrator', type: 'entity')}
        
        it 'works' do
          entity.update(cre_national_id_cristal: '000001', cre_national_email: user.email)
          rex_entity_json[:fields][:itiephycrenational] = ''

          subject

          expect(Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')).to be_nil
          expect(entity.reload.cre_national_id_cristal).to be_nil
          expect(entity.reload.cre_national_email).to be_nil
          expect(entity.reload.cre_national_first_name).to be_nil
          expect(entity.reload.cre_national_last_name).to be_nil
        end 
      end 

      describe 'removing cre national but still cre' do
        let!(:user_right) {create(:user_right, user_id: user2.id, entity_id: entity.id, role: 'cre_national', right: 'administrator', type: 'entity')}

        it 'works' do
          rex_entity_json[:fields][:itiephycrenational] = ''
          rex_contact_json[:fields][:id_etablissement_cristal] = [entity.id_cristal]
          allow(object.client).to receive(:search_contacts).and_return([rex_contact_json])
          entity.update(cre_national_id_cristal: '000002', cre_national_email: user2.email)

          subject

          user_right = Hermes::UserRight.find_by(entity_id: entity.id)
          expect(user_right.user_id).to eq(user2.id)
          expect(user_right.role).to eq('cre')

          expect(entity.reload.cre_national_id_cristal).to be_nil
          expect(entity.reload.cre_national_email).to be_nil
          expect(entity.reload.cre_national_first_name).to be_nil
          expect(entity.reload.cre_national_last_name).to be_nil
          expect(Hermes::UserRight.where(entity_id: entity.id, user: user2).count).to eq(1)
        end 
      end 

      describe 'removing cre national and missing from REX' do
        let!(:user_right) {create(:user_right, user_id: user2.id, entity_id: entity.id, role: 'cre_national', right: 'administrator', type: 'entity')}

        it 'works' do
          rex_entity_json[:fields][:itiephycrenational] = ''
          entity.update(cre_national_id_cristal: '000003', cre_national_email: user2.email)

          subject

          expect(Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')).to be_nil
          expect(entity.reload.cre_national_id_cristal).to be_nil
          expect(entity.reload.cre_national_email).to be_nil
          expect(entity.reload.cre_national_first_name).to be_nil
          expect(entity.reload.cre_national_last_name).to be_nil
        end 
      end 
    end 
  end 

  describe "#synchronise" do 
    let!(:support) { create(:user, email: 'support@plugandwork.fr' , auth_roles: ['superadministrateur']) }
    let!(:date) { '2024-02-15' }
    let!(:object) { described_class.new(date: date) }

    it 'works' do 
      VCR.use_cassette("rex_synchronise", :match_requests_on => [:method, :uri, :path]) do
        object.synchronise
      end

      expect(Hermes::Entity.all().count).to eq(22)
      expect(User.where(auth_roles: ['cre']).count).to eq(3)
      expect(User.where(auth_roles: 'cre').count).to eq(49)
      expect(User.where(auth_roles: 'collaborateur offre logement').count).to eq(46)

      last_created_doc = Doc.where(type: 'file').order_by(created_at: :desc).first
      expect(last_created_doc.title).to include('Synchronisation REX')

      last_created_email = Doc.where(type: 'email').order_by(created_at: :desc).first
      expect(last_created_email.title).to include('Rapport Synchronisation REX')
    end 

    it 'log errors' do 
      allow(object.client).to receive(:get_page).with({'traceability.lastModifiedDate' => ">#{date}"}, 'lessor', 1).and_call_original
      allow(object.client).to receive(:get_page).with({'traceability.lastModifiedDate' => ">#{date}"}, 'entity', 1).and_call_original
      allow(object.client).to receive(:get_page).with({'traceability.lastModifiedDate' => ">#{date}"}, 'entity', 2).and_return({error: 'Trigered error'})
      allow(object.client).to receive(:get_page).with({'traceability.lastModifiedDate' => ">#{date}"}, 'entity', 3).and_call_original

      5.times do |i| 
        allow(object.client).to receive(:get_page).with({'traceability.lastModifiedDate' => ">#{date}"}, 'contact', i + 1).and_call_original
      end 

      VCR.use_cassette("rex_synchronise", :match_requests_on => [:method, :uri, :path]) do
        object.synchronise
      end

      expect(Hermes::Entity.all().count).to eq(13)

      last_created_doc = Doc.where(type: 'file').order_by(created_at: :desc).first
      expect(last_created_doc.title).to include('Synchronisation REX')
      expect(last_created_doc.file.data).to include('Trigered error')

      last_created_email = Doc.where(type: 'email').order_by(created_at: :desc).first
      expect(last_created_email.title).to include('Rapport Synchronisation REX')
      expect(last_created_email.body).to include('Trigered error')
    end 
    
    it 'add error to the first page' do 
      allow(RestClient).to receive(:get).and_raise(RestClient::BadRequest)

      VCR.use_cassette("rex_synchronise", :match_requests_on => [:method, :uri, :path]) do
        object.synchronise
      end

      last_created_doc = Doc.where(type: 'file').order_by(created_at: :desc).first
      expect(last_created_doc.title).to include('Synchronisation REX')
      expect(last_created_doc.file.data).to include('Error : getting first_page')
    end 

    it 'when wrong token' do
      force_error = true
      original = RestClient.method(:get)

      allow(RestClient).to receive(:get) do |url, options|
        if force_error
          force_error = false
          raise(RestClient::Unauthorized)
        else
          original.call(url, options)
        end
      end
      expect(object.client).to receive(:get_token).twice.and_call_original

      VCR.use_cassette("rex_synchronise", :match_requests_on => [:method, :uri, :path], :allow_playback_repeats => true) do
        object.synchronise
      end

      last_created_doc = Doc.where(type: 'file').order_by(created_at: :desc).first
      expect(last_created_doc.file.data).not_to include('Error')
    end 

  end 
end